# Program to align two DNA Sequences using Needleman & Wunsch algorithm
#
# Reference:
# Needleman, S. B. & Wunsch, C. D. (1970) "A General Method Applicable 
# to the Search for Similarities in the Amino Acid Sequence of Two Proteins," 
# J. Mol. Biol., 48, 443-453.
#
# Dr. Walter F. de Azevedo Jr.
# azevedolab.net
# December, 7th 2017.
#
# Import library
import numpy as np

# Sequences to align
seq1 = "GAATTCAGTTA"
seq2 = "GGATCGA"

# Length of sequences
n1 = len(seq1)+1
n2 = len(seq2)+1

# Set up path matrix
M = np.array([[0]*n1]*n2,int)        # np.array([[0]*column]*row,int)

# Show sequence
print("Sequences to be aligned:")
for i in range(n1-1):
    print(seq1[i],end="")
print()
for j in range(n2-1):
    print(seq2[j],end="")

# Define path matrix
# Looping through matrix
for i in range(1,n2):
    for j in range(1,n1):
        if seq1[j-1] == seq2[i-1]:
            M[i][j] = 1

# Build path matrix      
for i in range(1,n2):
    for j in range(1,n1):
        my_array = [M[i-1][j-1] + M[i][j], M[i][j-1],M[i-1][j]]
        M[i][j] = np.max(my_array)

# Show path matrix
print("\n\nPath matrix:")
for i in range(n2):
    for j in range(n1):
        print(M[i][j],end=" ")
    print()

# Set up initial values for i and j
i = n2-1
j = n1-1

# Set up empty strings
s1 = ""
s2 = ""

# while loop
while( i > 0 or j >0):
    my_array = [M[i-1][j-1], M[i][j-1],M[i-1][j]]
    m1 = np.max(my_array)
    if m1 == M[i-1][j-1]:
        s1 += seq1[j-1]
        s2 += seq2[i-1]      
        i -= 1
        j -= 1
    elif m1 == M[i][j-1]:
        s1 += seq1[j-1]
        s2 += "-"
        j -= 1
    elif m1 == M[i-1][j]:
        s1 += "-"
        s2 += seq2[i-1]
        i -= 1

print("\nAligned sequences:")
print(s1[::-1])
print(s2[::-1])